#! /usr/bin/env python

# testprank.py
# 
# Description: A demonstration of fitting a mallows model from simulated *partial* rankings
#
# Author: Jonathan Huang

import mallows as ma
import permFuncs as pf
from math import exp

# instanstiate a generalized mallows model data structure and simulate from model
sigma0 = [3,2,0,1]
phi = [exp(-.8), exp(-.5), exp(-.7)]
alpha = .5

# set up generalized mallows model data structure
n = len(sigma0)
P = ma.mallows.createMallows(phi,sigma0)

# simulate partial rankings from model
numsamps = 100
sigmas = P.draw(numsamps)
Dprank = []
for sig in sigmas:
	Dprank.append(pf.randPrankCensor(sig,alpha))

# run EM for learning mallows model from partial rankings
numiter = 10
phi_init = (n-1)*[.001]
sig_init = range(n)
Pinit = ma.mallows.createMallows(phi_init,sig_init)
Pfit = ma.mallows.fitMallows_prank(n,Dprank,Pinit,numiter,verbosity = True)

print('\nModel parameters')
print('sigma0: ' + str(sigma0))
print('phi: ' + str(phi))
print('\nFit parameters')
print('estimated sigma0: ' + str(Pfit.sigma0))
print('estimated phi: ' + str(Pfit.phi))









